/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#ifndef __IO_DATAINPUT_INL
#define __IO_DATAINPUT_INL

namespace magnum { namespace io {

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DataInput::DataInput(InputStream* in) {
	ASSERT(in != NULL);
	m_Input = in;
	m_Input->refer();
	m_BytesRead = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DataInput::DataInput(InputStream& in) {
	m_Input = &in;
	m_Input->refer();
	m_BytesRead = 0;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline DataInput::~DataInput() {
	if (m_Input != NULL) {
		m_Input->release();
		m_Input = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned long DataInput::getBytesRead() const {
	return m_BytesRead;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataInput::skip(unsigned long o) {
	ASSERT(m_Input != NULL);
	m_BytesRead += m_Input->skip(o);
}	


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned long DataInput::read(void *buffer, unsigned long size) {
	ASSERT(m_Input != NULL);
	unsigned long r = m_Input->read(buffer,size);
	m_BytesRead += r;
	return r;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline unsigned long DataInput::available() {
	ASSERT(m_Input != NULL);
	return m_Input->available();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline void DataInput::close() {
	if (m_Input != NULL) {
		m_Input->release();
		m_Input = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline bool DataInput::readBoolean() {
	ASSERT(m_Input != NULL);
	bool b;
	m_BytesRead += m_Input->read(&b, sizeof(b));
	return b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline float DataInput::readFloat() {
	ASSERT(m_Input != NULL);
	float f;
	m_BytesRead += m_Input->read(&f, sizeof(f));
	return f;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline double DataInput::readDouble() {
	ASSERT(m_Input != NULL);
	double d;
	m_BytesRead += m_Input->read(&d, sizeof(d));
	return d;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline char DataInput::readChar() {
	ASSERT(m_Input != NULL);
	char b;
	m_BytesRead += m_Input->read(&b, sizeof(b));
	return b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline byte DataInput::readByte() {
	ASSERT(m_Input != NULL);
	byte b;
	m_BytesRead += m_Input->read(&b, sizeof(b));
	return b;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline word DataInput::readWord() {
	ASSERT(m_Input != NULL);
	word w;
	m_BytesRead += m_Input->read(&w, sizeof(w));
	return w;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
inline dword DataInput::readDword() {
	ASSERT(m_Input != NULL);
	dword d;
	m_BytesRead += m_Input->read(&d, sizeof(d));
	return d;
}

}; };

#endif
