/* 
  Magnum 
  Copyright (C) 2002-2005 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "debug.logger.h"
#include "io.sysfile.h"

#include "object.inl"
#include "base.string.inl"
#include "io.outputstream.inl"
#include "io.sysfile.inl"
#include "system.timestamp.inl"
#include "debug.logger.inl"


namespace magnum { namespace debug {
using magnum::io::SystemFile;

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Logger::Logger(OutputStream* stream) {
	if (stream != NULL) {
		m_Stream = stream;
		m_Stream->refer();
	} else {
		m_Stream = NULL;
	}
	m_File = NULL;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Logger::Logger(File* file) {
	if (file != NULL) {
		m_File = file;
		m_File->refer();
		m_Stream = file->getOutputStream();
		m_Stream->refer();
	} else {
		m_File = NULL;
		m_Stream = NULL;
	}
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Logger::Logger(const char* fname) {
	try {
		m_File = SystemFile::create(fname);
	} catch(...) {
		m_File = NULL;
	}
	if (m_File == NULL) {
		m_Stream = NULL;
		return;	
	}
	m_Stream = m_File->getOutputStream();
	m_Stream->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Logger::Logger(const Logger& lf) {
	m_Stream = lf.m_Stream;
	if (m_Stream != NULL)
		m_Stream->refer();
	m_File = lf.m_File;
	if (m_File != NULL)
		m_File->refer();
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
Logger::~Logger() {
	if (m_Stream != NULL) {
		m_Stream->release();
		m_Stream = NULL;
	}
	if (m_File != NULL) {
		m_File->release();
		m_File = NULL;
	}
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::writeLine(const char* str) {
	if (m_Stream == NULL)
		return;
	try {
		m_Timestamp.getCurrentTime();
		String line = "[";
		line += m_Timestamp.toString();
		line += "] -- ";
		line += str;
		line += "\n";
		m_Stream->write(line, line.getLength());
		m_Stream->flush();
	} catch(...) {
	}
}

/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void Logger::writeLine(const String& str) {
	if (m_Stream == NULL)
		return;
	try {
		m_Timestamp.getCurrentTime();
		String line = "[";
		line += m_Timestamp.toString();
		line += "] -- ";
		line += str;
		line += "\n";
		m_Stream->write(line, line.getLength());
		m_Stream->flush();
	} catch(...) {
	}
}

}; };
